import os
from exif import Image
from shapely import geometry
import geopandas as gpd
from tqdm import tqdm
from abcli import file
from abcli.modules import objects
from ferfereh.logger import logger


# https://medium.com/spatial-data-science/how-to-extract-gps-coordinates-from-images-in-python-e66e542af354
def decimal_coords(coords, ref):
    decimal_degrees = coords[0] + coords[1] / 60 + coords[2] / 3600
    if ref in ["S", "W"]:
        decimal_degrees = -decimal_degrees
    return decimal_degrees


def get_image_info(image_path):
    try:
        with open(image_path, "rb") as src:
            img = Image(src)
        if not img.has_exif:
            logger.info(f"{file.name_and_extension(image_path)}: no EXIF.")
            return False, {}
    except:
        logger.error(f"{file.name_and_extension(image_path)}: bad EXIF.")
        return False, {}

    try:
        coords = (
            decimal_coords(img.gps_latitude, img.gps_latitude_ref),
            decimal_coords(img.gps_longitude, img.gps_longitude_ref),
        )
    except AttributeError:
        logger.error(f"{file.name_and_extension(image_path)}: bad EXIF.")
        return False, {}

    try:
        datetime = img.datetime_original
    except:
        logger.warning(f"{file.name_and_extension(image_path)}: no datetime_original.")
        datetime = "unknown"

    return True, {
        "datetime": datetime,
        "coords": coords,
    }


def publish_coords(output_filename):
    object_id = os.getenv("FERFEREH_IMAGE_OBJECT")
    if not object_id:
        logger.info("FERFEREH_IMAGE_OBJECT not found, quitting.")
        return False

    list_of_images = [
        filename
        for filename in objects.list_of_files(object_id)
        if file.extension(filename) in ["jpg"]
    ]
    logger.info(f"{len(list_of_images)} image(s) found.")

    coords = []
    metadata = {keyword: [] for keyword in "datetime,picture".split(",")}
    fail_count = 0
    for filename in tqdm(list_of_images):
        success_, info = get_image_info(filename)
        if not success_:
            fail_count += 1
            continue

        coords += [
            geometry.Point(
                info["coords"][1],
                info["coords"][0],
            )
        ]
        metadata["picture"] += [
            "https://kamangir-public.s3.ca-central-1.amazonaws.com/{}/{}".format(
                object_id,
                file.name_and_extension(filename),
            )
        ]
        metadata["datetime"] += [info["datetime"]]

    if fail_count:
        logger.info(f"{fail_count} failure(s).")

    gdf = gpd.GeoDataFrame(metadata, geometry=coords)
    gdf.crs = "EPSG:4326"
    gdf.to_file(output_filename)
    logger.info(f"->{output_filename}")

    return True



import os
from PIL import Image
from abcli import file
from ferfereh.logger import logger


def cleanup(path):
    logger.info(f"ferfereh.clean({path})")

    for filename in file.list_of(os.path.join(path, "*.jpeg")):
        image = Image.open(filename)

        width, height = image.size
        if width == height:
            continue

        size = max(width, height)
        new_image = Image.new("RGB", (size, size), (0, 0, 0))

        left = (size - width) // 2
        top = (size - height) // 2
        new_image.paste(image, (left, top))

        new_image.save(filename)
        logger.info(f"-> {filename}")

    return True



import argparse
from ferfereh import NAME, VERSION, DESCRIPTION
from ferfereh.logger import logger


parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help="cleanup|description|publish_coords|version",
)
parser.add_argument(
    "--output_filename",
    type=str,
)
parser.add_argument(
    "--path",
    type=str,
)
args = parser.parse_args()

success = False
if args.task == "cleanup":
    from ferfereh.utils import cleanup

    success = cleanup(args.path)
elif args.task == "description":
    print(DESCRIPTION)
    success = True
elif args.task == "publish_coords":
    from ferfereh.coords import publish_coords

    success = publish_coords(args.output_filename)
elif args.task == "version":
    print(f"{NAME}-{VERSION}")
    success = True
else:
    logger.error(f"-{NAME}: {args.task}: command not found.")

if not success:
    logger.error(f"-{NAME}: {args.task}: failed.")
